/**
 * @file
 * VuoCable interface.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This interface description may be modified and distributed under the terms of the GNU Lesser General Public License (LGPL) version 2 or later.
 * For more information, see http://vuo.org/license.
 */

#ifndef VUOCABLE_HH
#define VUOCABLE_HH

#include "VuoBase.hh"
#include "VuoNode.hh"

class VuoCompilerCable;
class VuoRendererCable;

/**
 * Represents a connection from a node's output port to a node's input port.
 */
class VuoCable : public VuoBase<VuoCompilerCable,VuoRendererCable>
{
public:
	VuoCable(VuoNode * fromNode, VuoPort * fromPort, VuoNode * toNode, VuoPort * toPort, bool carriesData);

	VuoNode * getFromNode(void);
	VuoPort * getFromPort(void);
	VuoNode * getToNode(void);
	VuoPort * getToPort(void);
	bool getCarriesData(void);
	void setFrom(VuoNode *fromNode, VuoPort *fromPort);
	void setTo(VuoNode *toNode, VuoPort *toPort);
	string getDeclaration(void);

private:
	VuoNode * fromNode;
	VuoPort * fromPort;
	VuoNode * toNode;
	VuoPort * toPort;

	bool carriesData;
};

#endif // VUOCABLE_HH
