/**
 * @file
 * VuoCompilerCodeGenUtilities interface.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This interface description may be modified and distributed under the terms of the GNU Lesser General Public License (LGPL) version 2 or later.
 * For more information, see http://vuo.org/license.
 */

#ifndef VUOCOMPILERCODEGENUTILITIES_H
#define VUOCOMPILERCODEGENUTILITIES_H

#include "dispatch/dispatch.h"

/**
 * Handy methods for generating code.
 */
class VuoCompilerCodeGenUtilities
{
private:
	static PointerType * getDispatchSemaphoreType(Module *module);
	static PointerType * getDispatchGroupType(Module *module);
	static PointerType * getDispatchQueueType(Module *module);
	static StructType * getDispatchObjectElementType(Module *module);

	static bool isParameterPassedByValue(Function *function, int parameterIndex);

public:
	static StructType * getDispatchObjectType(Module *module);

	static GlobalVariable * generateAllocationForSemaphore(Module *module, string identifier);
	static void generateInitializationForSemaphore(Module *module, BasicBlock *block, GlobalVariable *semaphoreVariable, int initialValue=1);
	static void generateWaitForSemaphore(Module *module, BasicBlock *block, GlobalVariable *semaphoreVariable, dispatch_time_t timeout=DISPATCH_TIME_FOREVER);
	static void generateSignalForSemaphore(Module *module, BasicBlock *block, GlobalVariable *semaphoreVariable);

	static GlobalVariable * generateAllocationForDispatchGroup(Module *module, string identifier);
	static void generateInitializationForDispatchGroup(Module *module, BasicBlock *block, GlobalVariable *dispatchGroupVariable);
	static void generateSubmissionForDispatchGroup(Module *module, BasicBlock *block, GlobalVariable *dispatchGroupVariable, Function *workerFunction, Value *contextValue);
	static void generateWaitForDispatchGroup(Module *module, BasicBlock *block, GlobalVariable *dispatchGroupVariable, dispatch_time_t timeout=DISPATCH_TIME_FOREVER);

	static GlobalVariable * generateAllocationForDispatchQueue(Module *module, string identifier);
	static void generateInitializationForDispatchQueue(Module *module, BasicBlock *block, GlobalVariable *dispatchQueueVariable);
	static void generateSubmissionToDispatchQueue(Module *module, BasicBlock *block, GlobalVariable *dispatchQueueVariable, Function *workerFunction, Value *contextValue);

	static void generateFinalizationForDispatchObject(Module *module, BasicBlock *block, GlobalVariable *dispatchObjectVariable);

	static Value * generatePointerToValue(BasicBlock *block, Value *value);
	static Constant * generatePointerToConstantString(Module *module, string stringValue, string globalVariableName = "");
	static Constant * generatePointerToConstantArrayOfStrings(Module *module, vector<string> stringValues);
	static void generateStringMatchingCode(Module *module, Function *function, BasicBlock *initialBlock, BasicBlock *finalBlock, Value *inputStringValue, map<string, pair<BasicBlock *, BasicBlock *> > blocksForString);
	static Value * generateMemoryAllocation(Module *module, BasicBlock *block, Type *elementType, Value *elementCountValue);

	static Function * getStrcmpFunction(Module *module);
	static Function * getMallocFunction(Module *module);
	static Function * getInitializeReferenceCountsFunction(Module *module);
	static Function * getFinalizeReferenceCountsFunction(Module *module);
	static Function * getVuoRetainFunction(Module *module);
	static Function * getVuoReleaseFunction(Module *module);

	static Type * getParameterTypeBeforeLowering(Function *function, int parameterIndex, Module *module, string typeName);
	static Value * convertArgumentToParameterType(Value *argument, Function *function, int parameterIndex, Value **secondArgument, Module *module, BasicBlock *block);
	static Value * convertReturnValueToExpectedType(Value *returnValue, Type *expectedType, BasicBlock *block);
	static Value * callFunctionWithStructReturn(Function *function, vector<Value *> args, BasicBlock *block);
	static bool isFunctionReturningStructViaParameter(Function *function);
};

#endif
