/**
 * @file
 * VuoCompilerGenerator interface.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This interface description may be modified and distributed under the terms of the GNU Lesser General Public License (LGPL) version 2 or later.
 * For more information, see http://vuo.org/license.
 */

#ifndef VUOCOMPILERGENERATORACTION_H
#define VUOCOMPILERGENERATORACTION_H

#include "VuoCompilerGeneratorPort.hh"
#include "VuoCompilerNode.hh"


/**
 * The callback that executes when a generator port on a node generates an event.
 */
class VuoCompilerGeneratorAction
{
private:
	VuoCompilerGeneratorPort *port;
	VuoCompilerNode *node;
	GlobalVariable *dispatchQueueVariable;
	GlobalVariable *previousDataVariable;

public:
	VuoCompilerGeneratorAction(VuoCompilerGeneratorPort *port, VuoCompilerNode *node);
	void generateAllocation(Module *module);
	void generateInitialization(Module *module, BasicBlock *block);
	void generateFinalization(Module *module, BasicBlock *block);
	Function * generateSubmissionToDispatchQueue(Module *module, BasicBlock *block, string identifier);
	Value * generateDataValueDidChange(Module *module, BasicBlock *block, Function *generatorWorker);
	Type * getDataType(void);
	string getIdentifier(void);
};


#endif
