﻿/**
 * @file
 * VuoCompilerNodeArgumentClass interface.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This interface description may be modified and distributed under the terms of the GNU Lesser General Public License (LGPL) version 2 or later.
 * For more information, see http://vuo.org/license.
 */

#ifndef VUOCOMPILERNODEARGUMENTCLASS_H
#define VUOCOMPILERNODEARGUMENTCLASS_H

#include "VuoBaseDetail.hh"

#include "VuoPortClass.hh"

/**
 * A parameter to a node's event and/or init function. To create an argument to pass
 * for this parameter, instantiate a @c VuoCompilerNodeArgument.
 *
 * @see VuoPortClass
 */
class VuoCompilerNodeArgumentClass : public VuoBaseDetail<VuoPortClass>
{
protected:
	Type *type; ///< The parameter's type.
	bool inEventFunction; ///< Does this parameter appear in the node's event function?
	size_t indexInEventFunction; ///< Where this parameter appears in the node's event function's parameter list.
	bool inInitFunction; ///< Does this parameter appear in the node's init function?
	size_t indexInInitFunction; ///< Where this parameter appears in the node's init function's parameter list.

	VuoCompilerNodeArgumentClass(string name, VuoPortClass::PortType portType, Type *type,
								 bool inEventFunction, size_t indexInEventFunction,
								 bool inInitFunction, size_t indexInInitFunction);
	virtual ~VuoCompilerNodeArgumentClass(void);  ///< to make this class dynamic_cast-able

public:
	virtual Type * getType(void);
	bool isInEventFunction(void);
	size_t getIndexInEventFunction(void);
	void setIndexInEventFunction(size_t indexInEventFunction);
	bool isInInitFunction(void);
	size_t getIndexInInitFunction(void);
	void setIndexInInitFunction(size_t indexInInitFunction);
};


#endif
