/**
 * @file
 * VuoPortClass interface.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This interface description may be modified and distributed under the terms of the GNU Lesser General Public License (LGPL) version 2 or later.
 * For more information, see http://vuo.org/license.
 */

#ifndef VUOPORTCLASS_HH
#define VUOPORTCLASS_HH

#include "VuoBase.hh"

class VuoCompilerPortClass;
class VuoRendererPortClass;
class VuoCompilerNodeArgumentClass;

/**
 * A port type on a @c VuoNodeClass.
 *
 * @see VuoCompilerPortClass
 */
class VuoPortClass : public VuoBase<VuoCompilerNodeArgumentClass,void>
{
public:
	/**
	 * Possible port types.
	 */
	enum PortType
	{
		notAPort, ///< Represents a dummy port, or a @c VuoCompilerNodeArgumentClass subclass that isn't a port (such as instance data or port data).
		dataAndEventPort, ///< See @c #VuoInputData and @c #VuoOutputData.
		eventOnlyPort, ///< See @c #VuoInputConductor and @c #VuoOutputConductor, and @c #VuoInputReceptor.
		generatorPort ///< See @c #VuoOutputGenerator.
	};

	VuoPortClass(string name, enum PortType portType);

	string getName(void);

	PortType getPortType(void);

private:
	string name; ///< The parameter's name, used as an identifier and for display.

	enum PortType portType; ///< The port's type.
};

#endif // VUOPORTCLASS_HH
