/**
 * @file
 * VuoPublishedPort interface.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This interface description may be modified and distributed under the terms of the GNU Lesser General Public License (LGPL) version 2 or later.
 * For more information, see http://vuo.org/license.
 */

#ifndef VUOPUBLISHEDPORT_HH
#define VUOPUBLISHEDPORT_HH

#include <set>
#include "VuoBase.hh"
#include "VuoType.hh"

class VuoCompilerPublishedPort;

/**
 * This class represents a published port in a graph. Published ports are visible to clients
 * of the graph, for example to a @c VuoRunner running the graph or to another graph in which
 * this graph is a subgraph.
 *
 * @see VuoCompilerPublishedPort
 */
class VuoPublishedPort : public VuoBase<VuoCompilerPublishedPort,void>
{
public:
	VuoPublishedPort(string name, VuoType *type);
	string getName(void);
	VuoType * getType(void);

	friend class VuoRunner;

private:
	void setConnectedPortIdentifiers(set<string> connectedPortIdentifiers);
	set<string> getConnectedPortIdentifiers(void);

	string name;
	VuoType *type;
	set<string> connectedPortIdentifiers;
};

#endif // VUOPUBLISHEDPORT_HH
