/**
 * @file
 * VuoRendererCable interface.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This interface description may be modified and distributed under the terms of the GNU Lesser General Public License (LGPL) version 2 or later.
 * For more information, see http://vuo.org/license.
 */

#ifndef VUORENDERERCABLE_HH
#define VUORENDERERCABLE_HH

#include "VuoBaseDetail.hh"
#include "VuoCable.hh"
#include "VuoRendererItem.hh"

/**
 * Renders a cable in a @c VuoRendererGraph.
 */
class VuoRendererCable : public VuoRendererItem, public VuoBaseDetail<VuoCable>
{
public:
	VuoRendererCable(VuoCable * baseCable);

	QRectF boundingRect(void) const;
	QPainterPath shape(void) const;
	void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);
	void setPenToHighlightSelection(QPen *pen);
	void setPenToHighlightDisconnectionPotential(QPen *pen);
	void setFloatingEndpointLoc(QPointF loc);
	void removeFromScene();
	void highlightEligiblePorts(void);
	void extendedHoverEnterEvent(QPointF scenePos);
	void extendedHoverMoveEvent(QPointF scenePos);
	void extendedHoverLeaveEvent(QPointF scenePos);
	VuoPort * getFloatingEndpointPreviousToPort();
	void setFloatingEndpointPreviousToPort(VuoPort *port);
	void setEligibleForDisconnection(bool eligible);
	void updateGeometry(void);

protected:
	void mousePressEvent(QGraphicsSceneMouseEvent *event);

private:
	// Drawing configuration
	static const qreal cableWidth;
	QPointF floatingEndpointLoc;
	VuoPort *floatingEndpointPreviousToPort;
	bool isEligibleForDisconnection;

	// Cable attributes affecting drawing

	// Internal methods
	QPainterPath getCablePath(void) const;
	void drawPathAlongPath(QPainter *painter, QPainterPath chevron, QBrush brush, qreal scale, QPainterPath path, qreal percent);
};

#endif // VUORENDERERCABLE_HH
