/**
 * @file
 * VuoRendererGraph interface.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This interface description may be modified and distributed under the terms of the GNU Lesser General Public License (LGPL) version 2 or later.
 * For more information, see http://vuo.org/license.
 */

#ifndef VUORENDERERGRAPH_HH
#define VUORENDERERGRAPH_HH

#include "VuoCable.hh"

class VuoCompilerNode;
class VuoRendererNode;
class VuoRendererTypecastPort;
class VuoRendererCable;
class VuoCompilerGraphvizParser;
class VuoRendererSignaler;

/**
 * Provides a canvas upon which nodes and cables can be rendered.
 */
class VuoRendererGraph : public QGraphicsScene
{
public:
	VuoRendererGraph();

	void setParser(VuoCompilerGraphvizParser *parser);
	VuoRendererNode * createNode(VuoNode *node);
	VuoRendererNode * addNode(VuoNode *node);
	VuoRendererNode * addExistingNode(VuoRendererNode * rn);
	VuoRendererCable * createCable(VuoCable *c);
	VuoRendererCable * addCable(VuoCable *cable);
	VuoRendererCable * addExistingCable(VuoRendererCable * rc);
	void loadComposition(void);
	VuoRendererNode * getNodeNameTaken(string name);
	void collapseTypecastNodes(void);
	VuoRendererTypecastPort *  collapseTypecastNode(VuoRendererNode *rn);
	VuoRendererNode * uncollapseTypecastNode(VuoRendererTypecastPort *typecast);
	void removePortEligibilityHighlighting(void);
	void setAnyNodeHasBeenRepositioned(bool repositioned);
	bool getAnyNodeHasBeenRepositioned(void) const;

protected:
	VuoCompilerGraphvizParser *parser;
	map<string, VuoRendererNode *> nodeNameTaken;
	bool anyNodeHasBeenRepositioned;
	VuoRendererSignaler *signaler;

private:
	static void addFont(QString font);
};

#endif // VUORENDERERGRAPH_HH
