/**
 * @file
 * VuoRendererPort interface.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This interface description may be modified and distributed under the terms of the GNU Lesser General Public License (LGPL) version 2 or later.
 * For more information, see http://vuo.org/license.
 */

#ifndef VUORENDERERPORT_HH
#define VUORENDERERPORT_HH

#include "VuoRendererDrawer.hh"
#include "VuoRendererItem.hh"

#include "VuoBaseDetail.hh"
#include "VuoType.hh"

class VuoCompilerNodeClass;
class VuoCompilerNode;
class VuoRendererNode;
class VuoCable;
class VuoPort;
class VuoRendererSignaler;

/**
 * Renders a node's port in a @c QGraphicsScene.  Typically automatically created by a @c VuoRendererNode instance.
 */
class VuoRendererPort : public VuoRendererItem, public VuoBaseDetail<VuoPort>
{
public:
	VuoRendererPort(VuoPort *basePort, VuoRendererSignaler *signaler,
					bool isOutput, bool isReceptorPort=false, bool isFunctionPort=false);

	QRectF boundingRect(void) const;
	QPainterPath shape(void) const;
	void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);

	void setEligibleForConnection(bool eligible);
	bool canConnectTo(VuoRendererPort *toPort);
	bool isConnectedTo(VuoRendererPort *toPort);
	bool getInput(void) const;
	bool getOutput(void) const;
	bool getFunctionPort(void) const;
	bool hasConnectedCable(void) const;
	bool hasConnectedDataCable(void) const;
	bool grabRangeIncludes(QPointF scenePos);
	void updateGeometry();
	QPainterPath getPortPath(qreal inset, qreal drawerOffset) const;
	QRectF getPortRect(void) const;
//	VuoRendererDrawer * getAttachedDrawer(void) const; @@@
	static QPointF getPlugOffset(void);
	VuoType * getDataType(void) const;
	bool isConstant(void) const;
	string getConstantAsString(void) const;

	VuoRendererNode * getParentNode(void) const;

	VuoRendererPort * getTypecastParentPort();
	void setTypecastParentPort(VuoRendererPort *typecastParentPort);
	bool supportsDisconnectionByDragging(void);

	// Drawing configuration
	static const qreal portSpacing; ///< Vertical distance, in pixels at 1:1 zoom, between the center points of two ports.
	static QFont nodePortTitleFont; ///< Font used for rendering port titles and constant values.

protected:
	void mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event);
	void keyPressEvent(QKeyEvent *event);
	void hoverEnterEvent(QGraphicsSceneHoverEvent * event);
	void hoverLeaveEvent(QGraphicsSceneHoverEvent * event);

private:
	static const qreal portGrabRadius;

	// Drawing configuration
	static const qreal plugLength;
	static QFont nodePortDecorationFont;
	static QFont nodePortDecoration2Font;

	// Port attributes affecting drawing
	bool isOutput;
	bool isFunctionPort;
	bool isEligibleForConnection;
//	VuoRendererDrawer * attachedDrawer; @@@
	VuoRendererPort *typecastParentPort;
	VuoRendererSignaler *signaler;

	friend class TestVuoRenderer;

protected:
	bool isReceptorPort;

	QRectF getNameRect() const;

	QPainterPath getPortConstantPath(QRectF innerPortRect, QString text) const;
	QRectF getPortConstantTextRect(QString text, bool isConnected, bool isTypecast) const;
	QPainterPath getFunctionPortGlyph(void) const;

	void paintPortName(QPainter *painter);
};

#endif // VUORENDERERPORT_HH
