/**
 * @file
 * VuoSceneVertices C type definition.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VUOSCENEVERTICES_H
#define VUOSCENEVERTICES_H

#include "VuoPoint4d.h"

/**
 * @ingroup VuoTypes
 * @defgroup VuoSceneVertices VuoSceneVertices
 * Vertices representing a 3D object.
 *
 * @{
 */

/**
 * The way in which an @c elements array should be interpreted during rasterization.
 */
typedef enum {
	VuoSceneVertices_IndividualTriangles, ///< GL_TRIANGLES, requires `elementCount % 3 == 0`.
	VuoSceneVertices_TriangleStrip, ///< GL_TRIANGLE_STRIP, requires `elementCount >= 3`.
	VuoSceneVertices_TriangleFan ///< GL_TRIANGLE_FAN, requires `elementCount >= 3`.
} VuoSceneVertices_ElementAssemblyMethod;

/**
 * Vertices representing a 3D object.
 */
typedef struct
{
	unsigned long vertexCount; ///< Number of vertices in @c positions, @c normals, and @c textureCoordinates.
	/// @todo Change to `VuoDictionary_VuoPoint4D vertexAttributes;` once dictionaries exist: a string-keyed list of equal-size arrays of points (vertex attributes).  Typically contains at least 'positions', 'normals', and 'textureCoordinates'.
	VuoPoint4d * positions; ///< XYZW vertex positions
	VuoPoint4d * normals; ///< Vertex normals
	VuoPoint4d * textureCoordinates; ///< STRQ texture coordinates

	unsigned long elementCount; ///< Number of elements in @c elements.
	/**
	 * An array of size elementCount of integer elements (triangle indices) which are indexes into @c positions.
	 * (Requires conversion to i16 for OpenGL ES.)
	 */
	unsigned long * elements;

	/// The way in which the @c elements array should be interpreted during rasterization.
	VuoSceneVertices_ElementAssemblyMethod elementAssemblyMethod;
} VuoSceneVertices;

VuoSceneVertices VuoSceneVertices_valueFromString(const char * initializer);
char * VuoSceneVertices_stringFromValue(const VuoSceneVertices value);
char * VuoSceneVertices_summaryFromValue(const VuoSceneVertices value);

/**
 * @}
 */

#endif
